require("love_reactor/all")

Ball = class({
  sprite = love.graphics.newImage("sprites/ball.psd"),
  drag = {x = 500, y = 500}
}, {drawable, collidable, clickable, movable})

Ball.instances.aggregate("click_stream")
  .filter(function(e) return e.value.type == "down" end)
  .map(function(e)
    love.audio.newSource("sfx/clack.ogg"):play()
    e.member.velocity = vector.mult(vector.sub(e.member.position, {x = love.mouse.getX(), y = love.mouse.getY()}), 100)
  end)

Hole = class({
  sprite = love.graphics.newImage("sprites/ball.psd"),
  color = {0, 0, 0, 255}
}, {drawable, collidable})

BloodGod = class({
  sprite = love.graphics.newImage("sprites/blood_god.psd")
}, {drawable, clickable})


update_stream
  .map(function()
    for _, ball in pairs(Ball.instances.values()) do
      ball.x = ball.x % love.window.getWidth()
      ball.y = ball.y % love.window.getHeight()
    end
  end)

collision_stream
  .filter(function(e) return e.type == "start" end)
  .map_filter(collision_between(Ball.instances, Ball.instances))
  .map(function(e)
    love.audio.newSource("sfx/clock.ogg"):play()
    local v1 = e.a.velocity
    e.a.velocity = e.b.velocity
    e.b.velocity = v1
  end)

collision_stream
  .filter(function(e) return e.type == "start" end)
  .map_filter(collision_between(Ball.instances, Hole.instances))
  .map(function(e)
    love.audio.newSource("sfx/yay.mp3"):play()
    e.a.destroy()
  end)

update_stream
  .map(function()
    for _, ball in pairs(Ball.instances.values()) do
      ball.angular_velocity = vector.magnitude(ball.velocity) / 100
    end
  end)

function love.load()
  love.graphics.setBackgroundColor(colors.green)
  local ball_colors = shallow_clone(colors)
  table.removeValue(ball_colors, colors.black)
  for _, color in pairs(ball_colors) do
    Ball.new({
      x = math.random(1, love.window.getWidth()),
      y = math.random(1, love.window.getHeight()),
      color = color
    })
  end

  Hole.new({
    x = math.random(1, love.window.getWidth() - 100),
    y = math.random(1, love.window.getHeight() - 100)
  })

  local bg = BloodGod.new({
    x = love.window.getWidth() / 2,
    y = love.window.getHeight() / 2
  })
  bg.click_stream
    .map(function()
      love.audio.newSource("sfx/god.mp3"):play()
      update_stream
        .take_until(update_stream.delay(1))
        .map(function()
          bg.x = love.window.getWidth() / 2 + math.random(-10, 10)
          bg.y = love.window.getHeight() / 2 + math.random(-10, 10)
        end)
    end)
  tween(bg, "scale", {x=.2,y=.2}, {x=5, y=5}, 60, linear_interp, function()
    love.event.quit() 
  end)

  love.audio.newSource("sfx/song.ogg"):play()
end
